﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Data.OleDb;
using System.Globalization;
using System.Windows.Forms;

namespace Mango
{
    /// <summary>
    ///     ConnectionManager sert d'interface avec la base de données
    /// </summary>
    internal class ConnectionManager
    {
        private static readonly ConnectionManager instance = new ConnectionManager();

        private ConnectionManager()
        {
            Connection = new OleDbConnection(@"Provider=Microsoft.Jet.OLEDB.4.0;Data Source=..\..\bdEvents.mdb");
            Connection.Open();
        }


        public OleDbConnection Connection { get; }
        public DataSet DataSet { get; } = new DataSet();

        public static ConnectionManager GetInstance()
        {
            return instance;
        }

        /// <summary>
        ///     Met à jour la base de donnée locale pour une utilisation offline.
        /// </summary>
        public void FetchOffline()
        {
            DataSet.Clear();
            var tables =
                Connection.GetOleDbSchemaTable(OleDbSchemaGuid.Tables, new object[] {null, null, null, "TABLE"});

            for (var i = 0; i < tables.Rows.Count; i++)
            {
                var name = tables.Rows[i][2].ToString();
                var sql = "SELECT * FROM " + name;
                if (name == "Participants") sql = "select *, prenomPart + ' ' + nomPart as nomComp from Participants";
                var command = CreateCommand(sql);
                var adapter = new OleDbDataAdapter(command);
                adapter.Fill(DataSet, name);
            }
        }

        /// <summary>
        ///     Filtre les données locales du DataSet dans la table spécifiée
        /// </summary>
        /// <param name="table">La table où filtrer</param>
        /// <param name="filter">Le filtre</param>
        /// <returns>Une table contenant les éléments filtrés</returns>
        /// <exception cref="ArgumentException">Si il n'y a aucun résultat trouvé</exception>
        public DataTable Filter(string table, string filter)
        {
            var rows = DataSet.Tables[table].Select(filter);
            if (rows.Length == 0)
                throw new ArgumentException("Aucun résultat");
            return rows.CopyToDataTable();
        }

        /// <summary>
        ///     Crée une commande d'insertion pour les valeurs données dans la table
        /// </summary>
        /// <param name="table">La table ou insérer</param>
        /// <param name="values">Les paires identifiant / valeur</param>
        /// <returns>La commande créée</returns>
        public OleDbCommand InsertInto(string table, Dictionary<string, object> values)
        {
            var first = true;
            var sql = "INSERT INTO " + table + "(";
            foreach (var key in values.Keys)
            {
                if (!first) sql += ", ";
                else first = false;
                sql += key;
            }

            first = true;
            sql += ") VALUES (";
            foreach (var value in values.Values)
            {
                if (!first) sql += ", ";
                else first = false;

                if (value is string s)
                    sql += "\"" + s.Replace("\"", "\\\"") + "\"";
                else if (value is double d)
                    sql += d.ToString(CultureInfo.InvariantCulture);
                else if (value is DateTime t)
                    sql += t.ToString("d");
                else
                    sql += value.ToString();
            }

            sql += ")";
            return CreateCommand(sql);
        }

        /// <summary>
        ///     Crée une commande avec la requête et la connection active.
        /// </summary>
        /// <param name="sql">Requête SQL</param>
        /// <returns>Commande</returns>
        public OleDbCommand CreateCommand(string sql)
        {
            return new OleDbCommand(sql, Connection);
        }

        /// <summary>
        ///     Retourne tous les participants d'un événement
        /// </summary>
        /// <param name="eventId">L'id de l'événement</param>
        /// <returns>Les participants</returns>
        public DataTable GetParticipants(int eventId)
        {
            DataTable table = Filter("Invites", "codeEvent = " + eventId);

            var first = true;
            var users = "";
            foreach (DataRow row in table.Rows)
            {
                if (!first) users += ", ";
                else first = false;
                users += row["codePart"];
            }

            return Filter("Participants", "codeParticipant IN (" + users + ")");
        }
        
        /// <summary>
        /// Retourne toutes les dépenses dont l'utilisateur a bénéficié sur un événenement
        /// </summary>
        /// <param name="eventId">L'id de l'événement</param>
        /// <param name="userId">L'id de l'utilisateur</param>
        /// <returns>Toutes les dépenses bénéficiaires</returns>
        public DataTable GetDepensesBeneficiaire(int eventId, int userId)
        {
            DataTable codeDettes = Filter("Beneficiaires", "codePart = " + userId);
            bool first = true;
            string codes = "";
            foreach (DataRow row in codeDettes.Rows)
            {
                if (!first) codes += ", ";
                else first = false;
                codes += row["numDepense"];
            }

            return Filter("Depenses", "numDepense IN (" + codes + ") AND codeEvent = " + eventId);
        }

        /// <summary>
        /// Retourne les bénéficiaires d'une dépense
        /// </summary>
        /// <param name="depenseId">L'id de la dépense</param>
        /// <returns>Les bénéficiaires</returns>
        public DataTable GetBeneficiaires(int depenseId)
        {
            DataTable table = Filter("Beneficiaires", "numDepense = " + depenseId);
            bool first = true;
            string codes = "";
            foreach (DataRow row in table.Rows)
            {
                if (!first) codes += ", ";
                else first = false;
                codes += row["codePart"];
            }

            return Filter("Participants", "codeParticipant IN (" + codes + ")");
        }
        
        /// <summary>
        /// Retourne les participants n'ayant pas bénéficié d'une dépense
        /// </summary>
        /// <param name="depenseId">L'id de la dépense</param>
        /// <returns>Les non bénéficiaires</returns>
        public DataTable GetNonBeneficiaires(int depenseId, int eventId)
        {
            DataTable benefs = Filter("Beneficiaires", "numDepense = " + depenseId);
            bool first = true;
            string codes = "";
            foreach (DataRow row in benefs.Rows)
            {
                if (!first) codes += ", ";
                else first = false;
                codes += row["codePart"];
            }
            
            DataTable invites = Filter("Invites", "codeEvent = " + eventId);
            first = true;
            string codes2 = "";
            foreach (DataRow row in invites.Rows)
            {
                if (!first) codes2 += ", ";
                else first = false;
                codes2 += row["codePart"];
            }
            
            return Filter("Participants", "codeParticipant NOT IN (" + codes + 
                                          ") AND codeParticipant IN (" + codes2 + ")");
        }
    }
}