﻿using System;
using System.Data;
using System.Windows.Forms;
using iText.Kernel.Colors;
using iText.Kernel.Pdf;
using iText.Kernel.Pdf.Colorspace;
using iText.Layout;
using iText.Layout.Element;
using iText.Layout.Properties;
using HorizontalAlignment = iText.Layout.Properties.HorizontalAlignment;

namespace Mango
{
    public class PdfManager
    {
        
        private static ConnectionManager _connection = ConnectionManager.GetInstance();
        
        /// <summary>
        /// Génére les pdfs de bilan de l'événement à tous ses invités
        /// </summary>
        /// <param name="eventId">L'id de l'événement</param>
        /// <param name="output">Le nom du fichier de sortie (sans .pdf)</param>
        public static void GenererPdf(int eventId, string output)
        {
            var invites = _connection.Filter("Invites", "codeEvent = " + eventId);

            foreach (DataRow row in invites.Rows)
            {
                DataRow participant = GetParticipant((int) row["codePart"]);
                GenererPdf(eventId, (int) participant["codeParticipant"], output + "_" + participant["nomPart"] + ".pdf");
            }
        }

        /// <summary>
        /// Génére le pdf de bilan pour un événement à un utilisateur
        /// </summary>
        /// <param name="eventId">L'id de l'événement</param>
        /// <param name="userId">L'id de l'utilisateur</param>
        /// <param name="output">Le nom du fichier de sortie (avec .pdf)</param>
        private static void GenererPdf(int eventId, int userId, string output)
        {
            DataRow even = _connection.Filter("Evenements", "codeEvent = " + eventId).Rows[0];
            DataRow user = GetParticipant(userId);

            // Génération du pdf
            PdfDocument pdf = new PdfDocument(new PdfWriter(output));
            Document document = new Document(pdf);

            document.Add(new Paragraph()
                .Add(new Text("Récapitulatif de l'événement " + even["titreEvent"])
                    .SetFontColor(ColorConstants.ORANGE))
                .Add(new Text("\n  Date de début: " + DateTime.Parse(even["dateDebut"].ToString()).ToString("dd/MM/yyyy")))
                .Add(new Text("\n  Date de début: " + DateTime.Parse(even["datefin"].ToString()).ToString("dd/MM/yyyy")))
                .Add(new Text("\n  Description: " + even["description"]))
                .SetHorizontalAlignment(HorizontalAlignment.CENTER));

            document.Add(new Paragraph()
                .Add(new Text("\n\nRécapitulatif pour " + user["nomComp"])
                    .SetFontColor(ColorConstants.ORANGE))
                .Add("\n  Nombre de parts : " + user["nbParts"]));

            document.Add(CreateDepenses(eventId, userId));
            document.Add(CreateDepensesBeneficiaires(eventId, userId));
            document.Add(CreateDoitPayerA(eventId, userId));
            document.Add(CreateDoitRecevoirDe(eventId, userId));
            
            document.Close();
        }

        /// <summary>
        /// Crée la catégorie dépenses du bilan pour un utilisateur lors d'un événement
        /// </summary>
        /// <param name="eventId">L'id de l'événement</param>
        /// <param name="userId">L'id de l'utilisateur</param>
        /// <returns>Le paragraphe contenant la catégorie</returns>
        private static Paragraph CreateDepenses(int eventId, int userId)
        {
            var paragraph = new Paragraph();
            paragraph.Add(new Text("\n\nDépenses effectuées:\n"));

            Table table = CreateTable("Description");

            try
            {
                DataTable depenses = _connection.Filter("Depenses", 
                    "codePart = " + userId + " AND codeEvent = " + eventId);
                foreach (DataRow row in depenses.Rows)
                {
                    table.AddCell(new Cell().Add(new Paragraph(row["description"] as string)));
                    table.AddCell(new Cell().Add(new Paragraph(row["montant"] + " €")));
                }
            }catch(ArgumentException) { }

            paragraph.Add(table);
            return paragraph;
        }
        
        /// <summary>
        /// Crée la catégorie dépenses bénéficiaires du bilan pour un utilisateur lors d'un événement
        /// </summary>
        /// <param name="eventId">L'id de l'événement</param>
        /// <param name="userId">L'id de l'utilisateur</param>
        /// <returns>Le paragraphe contenant la catégorie</returns>
        private static Paragraph CreateDepensesBeneficiaires(int eventId, int userId)
        {
            Paragraph paragraph = new Paragraph();
            paragraph.Add(new Text("\nDépenses bénéficiaires:\n"));

            Table table = CreateTable("Description");

            try
            {
                DataTable benefices = _connection.GetDepensesBeneficiaire(eventId, userId);
                foreach (DataRow row in benefices.Rows)
                {
                    table.AddCell(new Cell().Add(new Paragraph(row["description"] as string)));
                    table.AddCell(new Cell().Add(new Paragraph(row["montant"] + " €")));
                }
            }catch(ArgumentException) { }

            paragraph.Add(table);
            return paragraph;
        }
        
        /// <summary>
        /// Crée la catégorie "doit payer à" du bilan pour un utilisateur lors d'un événement
        /// </summary>
        /// <param name="eventId">L'id de l'événement</param>
        /// <param name="userId">L'id de l'utilisateur</param>
        /// <returns>Le paragraphe contenant la catégorie</returns>
        private static Paragraph CreateDoitPayerA(int eventId, int userId)
        {
            Paragraph paragraph = new Paragraph();
            paragraph.Add(new Text("\nDoit payer à :\n"));

            Table table = CreateTable("Nom");

            try
            {
                DataTable data = _connection.Filter("BilanPart", 
                    "codeEvent = " + eventId + " AND codeDonneur = " + userId);
                
                foreach (DataRow row in data.Rows)
                {
                    table.AddCell(new Cell().Add(new Paragraph((string) GetParticipant((int) row["codeReceveur"])["nomComp"])));
                    table.AddCell(new Cell().Add(new Paragraph(row["montant"] + " €")));
                }
            }catch(ArgumentException) { }

            paragraph.Add(table);
            return paragraph;
        }
        
        /// <summary>
        /// Crée la catégorie "doit recevoir de" du bilan pour un utilisateur lors d'un événement
        /// </summary>
        /// <param name="eventId">L'id de l'événement</param>
        /// <param name="userId">L'id de l'utilisateur</param>
        /// <returns>Le paragraphe conte ant la catégorie</returns>
        private static Paragraph CreateDoitRecevoirDe(int eventId, int userId)
        {
            Paragraph paragraph = new Paragraph();
            paragraph.Add(new Text("\nDoit recevoir de :\n"));

            Table table = CreateTable("Nom");

            try
            {
                DataTable data = _connection.Filter("BilanPart", 
                    "codeEvent = " + eventId + " AND codeReceveur = " + userId);
                
                foreach (DataRow row in data.Rows)
                {
                    table.AddCell(new Cell().Add(new Paragraph((string) GetParticipant((int) row["codeDonneur"])["nomComp"])));
                    table.AddCell(new Cell().Add(new Paragraph(row["montant"] + " €")));
                }
            }catch(ArgumentException) { }

            paragraph.Add(table);
            return paragraph;
        }

        /// <summary>
        /// Crée un tableau pour le pdf
        /// </summary>
        /// <param name="colonneName">Le nom de la 1er colonne</param>
        /// <returns>Le tableau créé</returns>
        private static Table CreateTable(string colonneName)
        {
            Table table = new Table(UnitValue.CreatePercentArray(2)).UseAllAvailableWidth();
            table.AddCell(new Cell().Add(new Paragraph(colonneName))
                .SetBackgroundColor(ColorConstants.ORANGE));
            table.AddCell(new Cell().Add(new Paragraph("Montant"))
                .SetBackgroundColor(ColorConstants.ORANGE));
            return table;
        }

        /// <summary>
        /// Retourne les participants d'un événement
        /// </summary>
        /// <param name="userId">L'id de l'événement</param>
        /// <returns>Les participants</returns>
        private static DataRow GetParticipant(int userId)
        {
            return _connection.Filter("Participants", "codeParticipant = " + userId).Rows[0];
        }
        
    }
}